grammar OsgiHeader;

@parser::header {
package com.springsource.server.osgi.manifest.parse;

import java.util.Map;
import java.util.HashMap;
}

@parser::members {

private List<String> names;	
private Map<String, String> attributes;
private Map<String, String> directives = new HashMap<String, String>();
protected List<HeaderDeclaration> declarations = new ArrayList<HeaderDeclaration>();
    
protected void displayError(RecognitionException e){
}
    
}

@lexer::header {
package com.springsource.server.osgi.manifest.parse;

} 

@lexer::members {


}


@rulecatch {
catch(RecognitionException e) {
	input.consume();
	displayError(e);
	//throw e;
}
}

//  ############### HERE BEGINS THE MAIN RULE SECTION FOR THE PARSER ###############

header 	:	pkgDecl (',' pkgDecl) *
	;
	
pkgDecl
@init {
this.names = new ArrayList<String>();
this.attributes = new HashMap<String, String>();
this.directives = new HashMap<String, String>();
}
@after {
this.declarations.add(new ImmutableHeaderDeclaration(this.names, this.attributes, this.directives));
}	
	: packageNames  (';' parameter) *
	;

packageNames
	: packageName (';' packageName) * 
	;

// Additions for DynamicImport-Package ...
dynHeader 	:	dynPkgDecl (',' dynPkgDecl) *
	;
	
dynPkgDecl
@init {
this.names = new ArrayList<String>();
this.attributes = new HashMap<String, String>();
this.directives = new HashMap<String, String>();
}
@after {
this.declarations.add(new ImmutableHeaderDeclaration(this.names, this.attributes, this.directives));
}	
	: dynPackageNames  (';' parameter) *
	;

dynPackageNames
	: dynPackageName (';' dynPackageName) * 
	;
	
dynPackageName 
	: 	
		wildcardedPackageName |
		star |
		packageName
		
	;
// ... end of additions for DynamicImport-Package.

// Additions for Require-Bundle ...

requireBundleHeader	: bundleDescription (',' bundleDescription) *
					;
					
bundleDescription
@init {
this.names = new ArrayList<String>();
this.attributes = new HashMap<String, String>();
this.directives = new HashMap<String, String>();
}
@after {
this.declarations.add(new ImmutableHeaderDeclaration(this.names, this.attributes, this.directives));
}
					: symbolicName (';' parameter) *
					;
					
symbolicName		: identifier {this.names.add($identifier.text);}
					;

// ... end of additions for Require-Bundle

// Additions for Fragment-Host ...

fragmentHostHeader	: bundleDescription
					;

// ... end of additions for Fragment-Host

// Additions for Bundle-SymbolicName ...

bundleSymbolicNameHeader : bundleDescription
                   ;
                   
// ... end of additions for Bundle-SymbolicName

// Additions for Import-Bundle

importBundleHeader : bundleDescription (',' bundleDescription) *
                   ;
                   
// ... end of additions for Import-Bundle

// Additions for Import-Library

importLibraryHeader : bundleDescription (',' bundleDescription) *
                    ;
                    
// ... end of additions for Import-Library

// Additions for Library-SymbolicName ...

librarySymbolicNameHeader : bundleDescription
                   ;
                   
// ... end of additions for Library-SymbolicName

webContextParamsHeader : bundleDescription (',' bundleDescription) *
                    ;
                    
webFilterMappingsHeader : bundleDescription (',' bundleDescription) *
                    ;
                    
webServletsHeader : bundleDescription (',' bundleDescription) *
                    ;
                    
webErrorPagesHeader : webErrorPageDeclaration (',' webErrorPageDeclaration) *
                    ;
                    
webErrorPageDeclaration
@init {
this.names = new ArrayList<String>();
this.attributes = new HashMap<String, String>();
this.directives = new HashMap<String, String>();
}
@after {
this.declarations.add(new ImmutableHeaderDeclaration(this.names, this.attributes, this.directives));
}	
	: webErrorPages  (';' parameter) *
	;

webErrorPages
	: webErrorPage (';' webErrorPage) * 
	;
	
webErrorPage 
	: 	filePath {this.names.add($filePath.text);}	
	;
			
bundleActivationPolicyHeader 
@init {
    this.names = new ArrayList<String>();
    this.attributes = new HashMap<String, String>();
    this.directives = new HashMap<String, String>();
}
@after {
    this.declarations.add(new ImmutableHeaderDeclaration(this.names, this.attributes, this.directives));
}
    : activationPolicy (';' directive) *
    ;  
                         
activationPolicy
    : identifier {this.names.add($identifier.text);}
    ;
	
parameter 
	:	directive | attribute
	;
	
directive
	:	identifier ':=' argument {this.directives.put($identifier.text, $argument.text);}
	;

attribute
	:	identifier '=' argument {this.attributes.put($identifier.text, $argument.text);}
	;

argument
	: 	identifier | QUOTED_STRING
	;

/*
 Note that the JLS allows a diverse set of Unicode characters, such as Greek characters, in package names. This grammar is more restrictive.
 See [ENGINE-303].
 */
packageName 
	: packageIdentifier {this.names.add($packageIdentifier.text);}
	;

wildcardedPackageName
	: wildPackageIdentifier {this.names.add($wildPackageIdentifier.text);}
	;
	
star
	: STAR {this.names.add($STAR.text);}
	;

QUOTED_STRING
	:  '"' (~('\\' | '"' | '\r' | '\n') | '\\' ('\\' | '"')) * '"'
	{setText(getText().substring(1, getText().length()-1));}
	;
	
filePath
    : '/' * filePathPart ('/' filePathPart) *
    ;
    
filePathPart
    : (ALPHA | DIGIT | '-' | '_' | '.' | '+' | '!' | '\'' | '(' | ')') +
    ;
	
identifier
	: identifierPart ('.' identifierPart) *
	;
	
packageIdentifier
	: packageIdentifierPart ('.' packageIdentifierPart) *
	;
	
wildPackageIdentifier
	: packageIdentifierPart ('.' packageIdentifierPart) * DOTSTAR
	;
	
identifierPart
	: (ALPHA | DIGIT | '_' | '-') +
	;

packageIdentifierPart
	: (ALPHA | DIGIT | '_' | '$') +
	;

DIGIT	:	'0'..'9'
	;
	
ALPHA	:	'a'..'z' | 'A'..'Z'
	;
	
DOTSTAR : '.*'
	;
	
STAR : '*'
	;

WS	:	' '+ {skip();}
	;

	
